<?php
class ContentigoUtils extends Module 
{

	function ContentigoUtils() {
	}

	function module_init( $moduleNamespace) {
	}

    private static $keyCharMap = array(
        'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h',
        'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p',
        'q', 'r', 's', 't', 'u', 'v', 'w', 'x',
        'y', 'z', 'A', 'B', 'C', 'D', 'E', 'F',
        'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
        'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V',
        'W', 'X', 'Y', 'Z', '0', '1', '2', '3',
        '4', '5', '6', '7', '8', '9'
    );
	//calling function has responsibility to ensure uniqueness.
	static function generateRandomKey( $keyLength, $prefix = '') {
		$key = '';
		for($i = 0; $i < $keyLength; $i++) {
			$key .= ContentigoUtils::$keyCharMap[ rand( 0, 61)];
		}
		return $prefix . $key;
	}

	public static $charConversionMap = array(
	    'Š'=>'S', 'š'=>'s', 'Ð'=>'Dj','Ž'=>'Z', 'ž'=>'z', 'À'=>'A', 'Á'=>'A', 'Â'=>'A', 'Ã'=>'A', 'Ä'=>'A',
	    'Å'=>'A', 'Æ'=>'A', 'Ç'=>'C', 'È'=>'E', 'É'=>'E', 'Ê'=>'E', 'Ë'=>'E', 'Ì'=>'I', 'Í'=>'I', 'Î'=>'I',
	    'Ï'=>'I', 'Ñ'=>'N', 'Ò'=>'O', 'Ó'=>'O', 'Ô'=>'O', 'Õ'=>'O', 'Ö'=>'O', 'Ø'=>'O', 'Ù'=>'U', 'Ú'=>'U',
	    'Û'=>'U', 'Ü'=>'U', 'Ý'=>'Y', 'Þ'=>'B', 'ß'=>'Ss','à'=>'a', 'á'=>'a', 'â'=>'a', 'ã'=>'a', 'ä'=>'a',
	    'å'=>'a', 'æ'=>'a', 'ç'=>'c', 'è'=>'e', 'é'=>'e', 'ê'=>'e', 'ë'=>'e', 'ì'=>'i', 'í'=>'i', 'î'=>'i',
	    'ï'=>'i', 'ð'=>'o', 'ñ'=>'n', 'ò'=>'o', 'ó'=>'o', 'ô'=>'o', 'õ'=>'o', 'ö'=>'o', 'ø'=>'o', 'ù'=>'u',
	    'ú'=>'u', 'û'=>'u', 'ý'=>'y', 'ý'=>'y', 'þ'=>'b', 'ÿ'=>'y', 'ƒ'=>'f'
	);
	//turns arbitary text into a URL 'friendly' string (good for SEO):
	static function textToURLText( $toClean, $ampersandReplacement = 'and') {
		$toClean = rtrim( $toClean, '&');
	    $toClean = str_replace('&', '-' . $ampersandReplacement . '-', $toClean);
	    $toClean = strtr($toClean, ContentigoUtils::$charConversionMap);
	    $toClean = trim(preg_replace('/[^\w\d_ -]/si', '', $toClean));//remove all illegal chars
	    $toClean = str_replace(' ', '-', $toClean);
	    $toClean = str_replace('--', '-', $toClean);
	    $toClean = rtrim( $toClean, '-');
	    return $toClean;
	}

	//converts line-breaks to '<br>' and links to <a>
	static public function textToHTML( $text) {
		
		$lineBreakReplacementPattern = '/(\r\n|\n|\r)/i';
		$lineBreakReplacement = '<br>';
		$text = preg_replace( $lineBreakReplacementPattern, $lineBreakReplacement, $text);
		
		$urlReplacementPattern = '/(\b(https?|ftp|file):\/\/[-A-Z0-9+&@#\/%?=~_|!:,.;]*[-A-Z0-9+&@#\/%=~_|])/i';
		$urlReplacement = '<a href="$1" target=\"_blank\">$1</a>';
		$text = preg_replace( $urlReplacementPattern, $urlReplacement, $text);

		return $text;
	}

	//simply returns an array value without creating warnings if key not found etc
	//be careful as null could be return as the array value or if key not found
	static function arrayValue( $array, $key, $valueIfNotFound = null) {
		if( isset( $array) && is_array( $array) && isset( $key) && array_key_exists( $key, $array) ) {
			return $array[ $key];
		}
		return $valueIfNotFound;
	}

	//converts bytes to kb etc.
	static function convertBytes( $size)
	 {
	    $unit = array('b','kb','mb','gb','tb','pb');
	    return @round($size/pow(1024,($i=floor(log($size,1024)))),2).' '.$unit[$i];
	 }

	//truncates a string (if required) to maximum length of $width:
	static function truncate( $string, $length) {
		if(strlen($string) > $length) {
			$string = wordwrap($string, $length);
			$string = substr($string, 0, strpos($string, "\n"));
		}
		return $string;
	}

	static function isValidURL( $url) {
		return preg_match('|^http(s)?://[a-z0-9-]+(.[a-z0-9-]+)*(:[0-9]+)?(/.*)?$|i', $url);
	}

	//	http://foo.com		index.php
	static function urlProxy( $sourceUrl, $embeddedUrl) {
		$urlParts = parse_url( $sourceUrl);
		
		$contentURLPath = null;
		if( array_key_exists('path', $urlParts) ) {
			$contentURLPath = $urlParts['path'];		//	/foo/bar	(but could be "/foo/bar/page.html")			
		}

		$contentURLRoot = $urlParts['scheme'] . '://' . $urlParts['host'];

		if( isset( $contentURLPath) ) {
			
			$lastPathPart = mb_strrichr( $contentURLPath, '/');
			if( false !== mb_stripos( $lastPathPart, '.') ) {
				//we've got something like: "/foo.bar" - assume its a file name and remove it from $contentURLPath:
				$postitioOfLastPart = mb_strripos( $contentURLPath, $lastPathPart); //+1 so we keep the trailing '/'
				$contentURLPath = mb_substr( $contentURLPath, 0, $postitioOfLastPart);
			}
			
			//$lastPathPart = mb_strrichr( $contentURLPath, '/');
			//$contentURLPath = mb_substr( $contentURLPath, $lastPathPart);
			
		}

// wow! have to replace ../ with actual path eg: "../../image/pie.jpg"
/*
 		http://foo.com/bar/baz/
 		../img.jpg
 		http://foo.com/bar/img.jpg
 		
		./foo = foo

		../
*/
		$embeddedURLParts = explode( '/', $embeddedUrl);
		


		if( '/' === $embeddedUrl[0]) {
			// relative to url root:
			return $contentURLRoot . $embeddedUrl;
		} else if( './' === substr( $embeddedUrl, 0, 2) ) {
			
			$embeddedUrl = substr( $embeddedUrl, 2);
			//just remove it: relative:
			//must be a relative path: index.php:
			if( isset( $contentURLPath) ){
				//$contentURLPath = '/';
				$contentURLPath = rtrim( $contentURLPath, '/');
			}
			return $contentURLRoot . $contentURLPath . '/' . $embeddedUrl;
			
		} else if( '../' === substr( $embeddedUrl, 0, 3) ) {
			//traverse up path tree:
			//echo $contentURLPath;
			$pathParts = explode('/', trim( $contentURLPath,'/') );
//				var_dump( $pathParts);
//				$pathDepth = count( $pathParts);
			
			$pathRecurseAmount = substr_count($embeddedUrl, '../');
			$newPath = '';
			$loopCount = sizeof( $pathParts) - $pathRecurseAmount;
			if( $loopCount >= 0) {
				for( $i = 0; $i < $loopCount; $i++ ) {
					$newPath .= $pathParts[ $i] . '/';
				}
			}
			$contentURLPath = $newPath;
			$embeddedUrl = substr( $embeddedUrl, $pathRecurseAmount * 3);
			return $contentURLRoot . '/' . $contentURLPath . $embeddedUrl;
			
		} else if( 'http://' === substr( $embeddedUrl, 0, 7) || 'https://' === substr( $embeddedUrl, 0, 8) ) {
			
			//it's a complete url, leave it:
			return $embeddedUrl;
		} else {
			
			//must be a relative path: index.php:
			if( isset( $contentURLPath) ){
				//$contentURLPath = '/';
				$contentURLPath = rtrim( $contentURLPath, '/');
			}
			return $contentURLRoot . $contentURLPath . '/' . $embeddedUrl;
		}
	}

	
	//fancy name - but it replaces href and src locations back to the true source, not femtoo
	//kinda like 'un proxying'
	//also attempts to strip javascript
	static function doUnproxyGet( $url, $stripJavascript = false) {
		
		$rawContent = ContentigoUtils::doGet( $url);

		//file_put_contents('c:\raw.txt', $rawContent);

		if( $stripJavascript) {
			
			//BUG - DOES NOT STRIP: onclick="javascript:doSomething()" or href="javascript:doSomething()"

			//strip 'script' tags:		<script	</script>
			//quick way, just replace with a tag name that will be ignored 'replaced'
				//		$rawContent = str_ireplace('<script', '<femtoo', $rawContent);
				//		$rawContent = str_ireplace('</script>', '</femtoo>', $rawContent);
			$pattern = '/<script.*?<\/script>/is';
			$rawContent = preg_replace($pattern, '', $rawContent);	//SLOW???
		}		


		/* BASIC PROXY STUFF */
		
		$urlParts = parse_url( $url);

		if( array_key_exists('path', $urlParts) ) {
			$contentURLPath = $urlParts['path'];		//	/foo/bar	(but could be "/foo/bar/page.html")
		} else {
			$contentURLPath = '';	//is this correct?			
		}
		$contentURLRoot = $urlParts['scheme'] . '://' . $urlParts['host'];

		//if path is: "/foo/bar/page.html", change path to: "/foo/bar":			/test_pages/script.html
		//Logger::Log( $contentURLPath);
		if( strlen( $contentURLPath) > 0) {
			$lastPathPart = mb_strrichr( $contentURLPath, '/');
			if( false !== mb_stripos( $lastPathPart, '.') ) {
				//we've got something like: "/foo.bar" - assume its a file name and remove it from $contentURLPath:
				$postitioOfLastPart = mb_strripos( $contentURLPath, $lastPathPart); //+1 so we keep the trailing '/'
				$contentURLPath = mb_substr( $contentURLPath, 0, $postitioOfLastPart);
			}
			//Logger::Log( $contentURLPath);
		}

		//("|\') this just means " or '
		//replace all href="/" with href="http://www.foo.com/"
		$pattern = '/href=("|\')\/(.*?)("|\')/i';
		$replacement = 'href=${1}' . $contentURLRoot . '/${2}${3}';
		$rawContent = preg_replace($pattern, $replacement, $rawContent);	//SLOW???

		//replace all href="bananas" with href="http://www.foo.com/" (important to ignore external url: 'http://google.com')
		$pattern = '/href=("|\')(?!http|https:\/\/)(.*?)("|\')/i';	//ooooww, a negative lookahead
		$replacement = 'href=${1}' . $contentURLRoot . $contentURLPath . '/${2}${3}';
		$rawContent = preg_replace($pattern, $replacement, $rawContent);	//SLOW???

		//replace all src="/" with src="http://www.foo.com/" (for images)
		$pattern = '/src=("|\')\/(.*?)("|\')/i';
		$replacement = 'src=${1}' . $contentURLRoot . '/${2}${3}';
		$rawContent = preg_replace($pattern, $replacement, $rawContent);	//SLOW???

		//replace all src="bananas" with src="http://www.foo.com/" (important to ignore external url: 'http://google.com')
		$pattern = '/src=("|\')(?!http|https:\/\/)(.*?)("|\')/i';	//ooooww, a negative lookahead
		$replacement = 'src=${1}' . $contentURLRoot . $contentURLPath . '/${2}${3}';
		$rawContent = preg_replace($pattern, $replacement, $rawContent);	//SLOW???

/*
		//replace all frm action="/" with action="http://www.foo.com/"
		$pattern = '/action="\/(.*?\")/i';
		$replacement = 'action="' . $contentURLRoot . '/${1}';
		$rawContent = preg_replace($pattern, $replacement, $rawContent);	//SLOW???

		//replace all form action="bananas" with action="http://www.foo.com/" (important to ignore external url: 'http://google.com')
		$pattern = '/action="(?!http:\/\/)(.*?\")/i';	//ooooww, a negative lookahead
		$replacement = 'action="' . $contentURLRoot . $contentURLPath . '/${1}';
		$rawContent = preg_replace($pattern, $replacement, $rawContent);	//SLOW???
*/

		//file_put_contents('c:\parsed.txt', $rawContent);

		return $rawContent;
	}
		
	static function doGet( $url, $timeoutSecs = 30) {

		$result = null;
		//Logger::Log( $url);
		$ch = curl_init( $url);
		curl_setopt( $ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_AUTOREFERER, true);
		//curl_setopt($ch, CURLOPT_COOKIEFILE, "cookie.txt");
		//curl_setopt($ch, CURLOPT_COOKIEJAR, "cookie.txt");
		
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );    # required for https urls
		
		//spoof as firefox	
		$ff3UserAgent = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.9.1.7) Gecko/20091221 Firefox/3.5.7";
		curl_setopt( $ch, CURLOPT_USERAGENT, $ff3UserAgent);
				
		curl_setopt( $ch, CURLOPT_HTTPGET, true);
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt( $ch, CURLOPT_MAXREDIRS, 4);
		//curl_setopt( $ch, CURLOPT_FAILONERROR, 1); //we can handle 404 etc?
		curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 3); //
		curl_setopt( $ch, CURLOPT_TIMEOUT, $timeoutSecs); //we can be more generous here.

		//to handle gzipped response etc:
		curl_setopt( $ch, CURLOPT_ENCODING, '');

		$result = curl_exec( $ch);
		
		if( empty( $result)) {
			//problem
			$curlErrNo = curl_errno($ch);
			$curlError = curl_error($ch);		
			
			
			//send alert email:
			/*
			$subject = 'Alert: MVC>utils>doGet: content is empty';
			$message = 'CURL_ERR_NO: ' . $curlErrNo;
			$message .= ' CURL_ERROR: ' . $curlError;
			$message .= ' URL: ' . $url;
			$model->sendAlertEmail( $subject, $message);			
			*/
		} else {
			//ok, we have a result.
			$info = curl_getinfo( $ch);
			$httpCode = $info['http_code'];
		}
	
		//IMPORTANT - dont 'return' without calling this:
		curl_close( $ch);
	
		return $result;
	}


	//this version adds and url params to the postDataArray
	//I think the above comment means url params are broken out and added 
	//as POST params.
	static function doPost2( $url, $postDataArray = null) {
		$result = null;
		
		$urlParts = parse_url( $url);
		$parsedURLParams = array();
		if( array_key_exists('query', $urlParts) && !empty( $urlParts['query']) ) {
			parse_str( $urlParts['query'], $parsedURLParams);			
		}

		if( empty( $postDataArray) ) {
			if( empty( $parsedURLParams) ) {
				$postDataArray = array();
			} else {
				$postDataArray = $parsedURLParams;
			}
		} else {
			if( empty( $parsedURLParams) ) {
				//$postDataArray = array();
			} else {
				$postDataArray = $postDataArray + $parsedURLParams;
				//var_dump( $postDataArray);
			}
		}
		
		//var_dump( $postDataArray);
		
		//strip any 'query' part from original URL:
		//$url = $urlParts['']
		if( false !== $pos = strpos($url, '?') ) {
			//we have a '?', do a substr:
			$url = substr( $url, 0, $pos);
		}
		
		
		$ch = curl_init( $url);
		curl_setopt( $ch, CURLOPT_HEADER, 0);
		curl_setopt( $ch, CURLOPT_POST, 1);
		curl_setopt ($ch, CURLOPT_POSTFIELDS, http_build_query( $postDataArray) );
//		curl_setopt ($ch, CURLOPT_POSTFIELDS,  $postDataArray );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 4);
		curl_setopt( $ch, CURLOPT_TIMEOUT, 10);
		curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt( $ch, CURLOPT_MAXREDIRS, 4);
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false);    # required for https urls

		//to handle gzipped response etc:
		curl_setopt( $ch, CURLOPT_ENCODING, '');

		$result = curl_exec( $ch);

		if( empty( $result)) {
			//problem
			//$curlErrNo = curl_errno($ch);
			//$curlError = curl_error($ch);		
			/*
			if( 7 == $curlErrNo) {
				//"couldn't connect to host"
				//Publisher not running? or wrong URL (port number etc)
			}*/
			
		} else {
			//ok, we have a result.
			//$info = curl_getinfo( $ch);
			//$httpCode = $info['http_code'];
		}

		curl_close( $ch);

		return $result;
	}
	
	static function doPost( $url, $postDataArray) {
		
		$result = null;
		
		$ch = curl_init( $url);
		curl_setopt( $ch, CURLOPT_HEADER, 0);
		curl_setopt( $ch, CURLOPT_POST, 1);
		curl_setopt ($ch, CURLOPT_POSTFIELDS, http_build_query( $postDataArray) );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1);

		//to handle gzipped response etc:
		curl_setopt( $ch, CURLOPT_ENCODING, '');

		$result = curl_exec( $ch);

		if( empty( $result)) {
			//problem
			$curlErrNo = curl_errno($ch);
			$curlError = curl_error($ch);		
			
			if( 7 == $curlErrNo) {
				//"couldn't connect to host"
				//Publisher not running? or wrong URL (port number etc)
			}
			
		} else {
			//ok, we have a result.
			$info = curl_getinfo( $ch);
			$httpCode = $info['http_code'];
		}
	
		//IMPORTANT - dont 'return' without calling this:
		curl_close( $ch);
	
		return $result;
	}


// multibyte string split

function mbStringToArray ($str) {
    if (empty($str)) return false;
    $len = mb_strlen($str);
    $array = array();
    for ($i = 0; $i < $len; $i++) {
        $array[] = mb_substr($str, $i, 1);
        }
    return $array;
    }
/*
// removes $rem at both ends
function mb_trim ($str, $rem = ' ') {
    if (empty($str)) return false;
    // convert to array
    $arr = Utils::mbStringToArray($str);
    $len = count($arr);
    // left side
    for ($i = 0; $i < $len; $i++) {
        if ($arr[$i] === $rem) $arr[$i] = '';
        else break;
        }
    // right side
    for ($i = $len-1; $i >= 0; $i--) {
        if ($arr[$i] === $rem) $arr[$i] = '';
        else break;
        }
    // convert to string
    return implode ('', $arr);
    }
    */

    static function mb_trim($string, $charlist='\\\\s', $ltrim=true, $rtrim=true)
    {
        $both_ends = $ltrim && $rtrim;

        $char_class_inner = preg_replace(
            array( '/[\^\-\]\\\]/S', '/\\\{4}/S' ),
            array( '\\\\\\0', '\\' ),
            $charlist
        );

        $work_horse = '[' . $char_class_inner . ']+';
        $ltrim && $left_pattern = '^' . $work_horse;
        $rtrim && $right_pattern = $work_horse . '$';

        if($both_ends)
        {
            $pattern_middle = $left_pattern . '|' . $right_pattern;
        }
        elseif($ltrim)
        {
            $pattern_middle = $left_pattern;
        }
        else
        {
            $pattern_middle = $right_pattern;
        }

        return preg_replace("/$pattern_middle/usSD", '', $string);
    } 

	static function mb_substr_replace($output, $replace, $posOpen, $posClose) {
        return mb_substr($output, 0, $posOpen).$replace.mb_substr($output, $posClose+1);
    } 
	

	static function startsWith($haystack, $needle)	{
	    return !strncmp($haystack, $needle, strlen($needle));
	}
	
	static function endsWith($haystack, $needle) {
	    $length = strlen($needle);
	    if ($length == 0) {
	        return true;
	    }
	
	    return (substr($haystack, -$length) === $needle);
	}
	
	//very useful util function - downloads a url directly to a file:
	static function downloadToFile( $url, $fileName, $maxDownloadKb = null) {

//THIS MUST ONLY DOWNLOAD USER LIMIT!

		$file = fopen( $url, "rb");
		if( $file) {
			$newf = fopen( $fileName, "wb");
	
			if( $newf)
			while( !feof( $file)) {
				fwrite($newf, fread( $file, 1024 * 8 ), 1024 * 8);
			}
		}
	
		if( $file) {
			fclose( $file);
		}
	
		if( $newf) {
			fclose( $newf);
		}
	}
	
}
?>